@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2013, KNOPFLERFISH project
+ * Copyright (c) 2010-2016, KNOPFLERFISH project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,6 +41,7 @@ import java.security.ProtectionDomain;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.ConcurrentModificationException;
 import java.util.Dictionary;
 import java.util.Enumeration;
@@ -56,12 +57,14 @@ import java.util.Set;
 import java.util.Vector;
 
 import org.knopflerfish.framework.Util.HeaderEntry;
+
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleException;
 import org.osgi.framework.Constants;
 import org.osgi.framework.Version;
 import org.osgi.framework.hooks.bundle.CollisionHook;
 import org.osgi.framework.namespace.ExecutionEnvironmentNamespace;
+import org.osgi.framework.namespace.NativeNamespace;
 import org.osgi.framework.wiring.BundleCapability;
 import org.osgi.framework.wiring.BundleRevision;
 
@@ -185,7 +188,11 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
 
   final BundleRevisionImpl bundleRevision;
 
-  final private BundleCapability identity;
+  final BundleCapabilityImpl identity;
+  final BundleNameVersionCapability bundleCapability;
+  final BundleNameVersionCapability hostCapability;
+
+  final NativeRequirement nativeRequirement;
 
   /**
    * Construct a new BundleGeneration for the System Bundle.
@@ -214,7 +221,11 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
     bundleRevision = new BundleRevisionImpl(this);
     classLoader = b.getClassLoader();
     processCapabilities(capabilityStr);
+    capabilities.put(NativeNamespace.NATIVE_NAMESPACE, Collections.singletonList(new BundleCapabilityImpl(this, b.fwCtx.props)));
     identity = new BundleCapabilityImpl(this);
+    bundleCapability = new BundleNameVersionCapability(this, BundleRevision.BUNDLE_NAMESPACE);
+    hostCapability = new BundleNameVersionCapability(this, BundleRevision.HOST_NAMESPACE);
+    nativeRequirement = null;
   }
 
 
@@ -401,6 +412,15 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
       symbolicNameParameters = null;
     }
     identity = symbolicName != null ? new BundleCapabilityImpl(this) : null;
+    if (v2Manifest && fragment==null) {
+      bundleCapability = new BundleNameVersionCapability(this, BundleRevision.BUNDLE_NAMESPACE);
+      hostCapability = !attachPolicy.equals(Constants.FRAGMENT_ATTACHMENT_NEVER) ? new BundleNameVersionCapability(this, BundleRevision.HOST_NAMESPACE) : null;
+    } else {
+      bundleCapability = null;
+      hostCapability = null;
+    }
+    final String nc = archive.getAttribute(Constants.BUNDLE_NATIVECODE);
+    nativeRequirement = nc != null ? new NativeRequirement(this, nc) : null;
   }
 
 
@@ -430,6 +450,9 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
     classLoader = null;
     bundleRevision = null;
     identity = null;
+    bundleCapability = null;
+    hostCapability = null;
+    nativeRequirement = null;
   }
 
 
@@ -532,16 +555,18 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
   /**
    *
    */
-  boolean resolvePackages(BundleImpl[] triggers) throws BundleException {
+  boolean resolvePackages(BundleImpl[] triggers, boolean isResolve) throws BundleException {
     ArrayList<BundleGeneration> detached = null;
-    attachFragments();
+    if (!isResolve) {
+      attachFragments();
+    }
     while (true) {
       if (bpkgs.resolvePackages(triggers)) {
         if (detached != null) {
           // TODO should we report fragment that failed to attach
           for (int i = detached.size() - 2; i >= 0; i--) {
             final BundleGeneration bg = detached.get(i);
-            if (bg.bundle.attachToFragmentHost(this)) {
+            if (bg.bundle.attachToFragmentHost(this, false)) {
               fragments.add(bg);
             }
           }
@@ -550,7 +575,7 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
 
         return true;
       }
-      if (isFragmentHost()) {
+      if (!isResolve && isFragmentHost()) {
         if (bundle.fwCtx.debug.resolver) {
           bundle.fwCtx.debug.println("Resolve failed, remove last fragment and retry");
         }
@@ -609,7 +634,7 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
         // retrieve all fragments this bundle host
         for (final BundleGeneration fbg : hosting) {
           // TODO, do we need to clean in case of BundleException?
-          fbg.bundle.attachToFragmentHost(this);
+          fbg.bundle.attachToFragmentHost(this, false);
         }
       }
     }
@@ -620,7 +645,7 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
    * Attaches a fragment to this bundle generation.
    * @throws BundleException 
    */
-  void attachFragment(BundleGeneration fragmentBundle) throws BundleException {
+  void attachFragment(BundleGeneration fragmentBundle, boolean isResolve) throws BundleException {
     if (attachPolicy.equals(Constants.FRAGMENT_ATTACHMENT_NEVER)) {
       throw new IllegalStateException("Bundle does not allow fragments to attach");
     }
@@ -628,16 +653,17 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
         && bundle.isResolved()) {
       throw new IllegalStateException("Bundle does not allow fragments to attach dynamically");
     }
-    try {
-      BundleClassPath.checkNativeCode(fragmentBundle.archive, bundle.fwCtx.props, true);
-    } catch (BundleException e) {
-      throw new IllegalStateException(e.getMessage());
+    if (fragmentBundle.nativeRequirement != null) {
+      try {
+        fragmentBundle.nativeRequirement.checkNativeCode();
+      } catch (BundleException e) {
+        throw new IllegalStateException(e.getMessage());
+      }
     }
-    if (!bundle.fwCtx.resolverHooks.filterMatches(fragmentBundle.fragment,
-                                                  new BundleNameVersionCapability(this, BundleRevision.HOST_NAMESPACE))) {
-      throw new IllegalStateException("Resolver hooks vetoed attach to: " + this);      
+    if (!bundle.fwCtx.resolverHooks.filterMatches(fragmentBundle.fragment, hostCapability)) {
+      throw new IllegalStateException("Resolver hooks vetoed attach to: " + this);
     }
-    final String failReason = bpkgs.attachFragment(fragmentBundle.bpkgs);
+    final String failReason = bpkgs.attachFragment(fragmentBundle.bpkgs, isResolve);
     if (failReason != null) {
       throw new IllegalStateException("Failed to attach: " + failReason);
     }
@@ -648,8 +674,6 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
         // TODO, should we unregister fragment packaaes
         throw new IllegalStateException(be.getMessage());
       }
-    } else {
-      
     }
     if (bundle.fwCtx.debug.resolver) {
       bundle.fwCtx.debug.println("Fragment(" + fragmentBundle.bundle + ") attached to host(id="
@@ -707,7 +731,7 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
       final Vector<BundleGeneration> fix = (Vector<BundleGeneration>)fragments.clone();
       for (final BundleGeneration bundleGeneration : fix) {
         final BundleImpl fb = bundleGeneration.bundle;
-        fb.getUpdatedState(null);
+        fb.getUpdatedState(null, false);
       }
     }
   }
@@ -1105,24 +1129,6 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
   }
 
 
-  BundleCapability getHostCapability()
-  {
-    if (v2Manifest
-        && !attachPolicy.equals(Constants.FRAGMENT_ATTACHMENT_NEVER) && fragment == null) {
-      return new BundleNameVersionCapability(this,
-                                             BundleRevision.HOST_NAMESPACE);
-    }
-    return null;
-  }
-
-  BundleNameVersionCapability getBundleCapability() {
-    if (v2Manifest && fragment==null) {
-      return new BundleNameVersionCapability(this, BundleRevision.BUNDLE_NAMESPACE);
-    }
-    return null;
-  }
-
-
   Vector<URL> getBundleClassPathEntries(final String name, final boolean onlyFirst) {
     BundleClassPath bcp = unresolvedBundleClassPath;
     if (bcp == null) {
@@ -1259,9 +1265,4 @@ public class BundleGeneration implements Comparable<BundleGeneration> {
     return res;
   }
 
-
-  BundleCapability getIdentityCapability() {
-    return identity;
-  }
-
 }
