@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2015, KNOPFLERFISH project
+ * Copyright (c) 2009-2016, KNOPFLERFISH project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,6 +36,7 @@ package org.knopflerfish.framework;
 
 import java.net.URLClassLoader;
 import java.util.Dictionary;
+import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.Locale;
 import java.util.Map;
@@ -149,6 +150,17 @@ public class FWProps {
    */
   public static final String KEY_KEYS = "org.knopflerfish.framework.bundleprops.keys";
 
+  /**
+   * Set to true if resolver should in case that no uses directive is specified
+   * implicitly check all imported packages for class conflicts.
+   */
+  public final static String RESOLVER_IMPLICIT_USES = "org.knopflerfish.framework.resolver.implicituses";
+
+  /**
+   * Set to true if resolver should provide system bundle packages from start.
+   */
+  public final static String RESOLVER_PREFER_SB = "org.knopflerfish.framework.resolver.prefersystembundle";
+
   /**
    * Common true string.
    */
@@ -178,13 +190,13 @@ public class FWProps {
   /**
    * The properties for this framework instance.
    */
-  protected Map<String, String> props = new Hashtable<String, String>();
+  protected Hashtable<String, String> props = new Hashtable<String, String>();
 
   /**
    * The default properties for this framework instance. TBD, maybe we should
    * make this JVM global!?
    */
-  protected Map<String, String> props_default = new Hashtable<String, String>();
+  protected Hashtable<String, String> props_default = new Hashtable<String, String>();
 
   // If set to true, then during the UNREGISTERING event the Listener
   // can use the ServiceReference to receive an instance of the service.
@@ -380,8 +392,12 @@ public class FWProps {
       ee.append(",OSGi/Minimum-1.1");
       ee.append(",OSGi/Minimum-1.2");
       // Set up the default ExecutionEnvironment
-      if (1 == javaVersionMajor) {
-        for (int i = javaVersionMinor; i > 1; i--) {
+      if (javaVersionMajor > 0) {
+        if (javaVersionMajor >= 9) {
+          ee.append(",JavaSE-").append(javaVersionMajor);
+        }
+        int maxMinor = javaVersionMajor == 1 ? javaVersionMinor : 8;
+        for (int i = maxMinor; i > 1; i--) {
           ee.append((i > 5) ? ",JavaSE-1." : ",J2SE-1.");
           ee.append(i);
         }
@@ -497,10 +513,17 @@ public class FWProps {
     setPropertyDefault(SERVICE_PERMISSIONADMIN_PROP, TRUE);
     setPropertyDefault(SYSTEM_PACKAGES_BASE_PROP, "");
     setPropertyDefault(SYSTEM_PACKAGES_FILE_PROP, "");
-    setPropertyDefault(SYSTEM_PACKAGES_VERSION_PROP, Integer.toString(javaVersionMajor) + "."
-        + javaVersionMinor);
-    setPropertyDefault(IS_DOUBLECHECKED_LOCKING_SAFE_PROP, javaVersionMajor >= 1
-        && javaVersionMinor >= 5 ? TRUE : FALSE);
+    StringBuffer pver = new StringBuffer();
+    if (javaVersionMajor >= 0) {
+      pver.append(javaVersionMajor);
+      if (javaVersionMinor >= 0) {
+        pver.append('.').append(javaVersionMinor);
+      }
+    } else {
+      pver.append(0);
+    }
+    setPropertyDefault(SYSTEM_PACKAGES_VERSION_PROP, pver.toString());
+    setPropertyDefault(IS_DOUBLECHECKED_LOCKING_SAFE_PROP, TRUE);
     setPropertyDefault(LDAP_NOCACHE_PROP, FALSE);
     setPropertyDefault(MAIN_CLASS_ACTIVATION_PROP, "");
     setPropertyDefault(STRICTBOOTCLASSLOADING_PROP, FALSE);
@@ -511,6 +534,13 @@ public class FWProps {
     setPropertyDefault(STARTLEVEL_COMPAT_PROP, FALSE);
     setPropertyDefault(STARTLEVEL_USE_PROP, TRUE);
     setPropertyDefault(READ_ONLY_PROP, FALSE);
+    setPropertyDefault(RESOLVER_IMPLICIT_USES, FALSE);
+    setPropertyDefault(RESOLVER_PREFER_SB, FALSE);
+  }
+
+
+  Map<String,Object> getFWProperties() {
+    return new HashMap(props);
   }
 
 }
\ No newline at end of file
