@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2015, KNOPFLERFISH project
+ * Copyright (c) 2003-2017, KNOPFLERFISH project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -98,6 +98,12 @@ class Resolver {
    */
   private HashMap<RequireBundle, BundlePackages> tempRequired = null;
 
+  /**
+   * Temporary map of attached fragment bundle packages done during a resolve
+   * operation.
+   */
+  private HashMap<BundlePackages, List<BundlePackages>> tempAttached = null;
+
   /**
    * Temporary set of package providers that are black listed in the resolve
    * operation.
@@ -151,6 +157,9 @@ class Resolver {
     if (null != tempRequired) {
       tempRequired.clear();
     }
+    if (null != tempAttached) {
+      tempAttached.clear();
+    }
     if (null != tempBlackList) {
       tempBlackList.clear();
     }
@@ -226,6 +235,7 @@ class Resolver {
       tempResolved = new HashSet<BundleGeneration>();
       tempProvider = new HashMap<String, ExportPkg>();
       tempRequired = new HashMap<RequireBundle, BundlePackages>();
+      tempAttached = new HashMap<BundlePackages, List<BundlePackages>>();
       tempWires = new ArrayList<BundleWireImpl>();
       tempBlackList = new HashSet<ExportPkg>();
       tempBackTracked = new HashSet<BundlePackages>();
@@ -248,6 +258,7 @@ class Resolver {
         tempBlackList = null;
         tempProvider = null;
         tempRequired = null;
+        tempAttached = null;
         tempWires = null;
         tempResolved = null;
         resolveThread = null;
@@ -385,6 +396,13 @@ class Resolver {
       if (!addTempResolved(bg)) {
         res = RESOLVER_HOOK_VETO;
       }
+      if (bg.isFragmentHost()) {
+        for (BundleGeneration fbg : bg.fragments) {
+          if (!addTempResolved(fbg)) {
+            res = RESOLVER_HOOK_VETO;
+          }
+        }
+      }
     } catch (BundleException be) {
       tempResolved = null;
       resolveThread = null;
@@ -409,7 +427,17 @@ class Resolver {
     tempBlackList = new HashSet<ExportPkg>();
     tempProvider = new HashMap<String, ExportPkg>();
     tempRequired = new HashMap<RequireBundle, BundlePackages>();
+    tempAttached = new HashMap<BundlePackages, List<BundlePackages>>();
     tempWires = new ArrayList<BundleWireImpl>();
+    if (bg.bundle == framework.systemBundle && framework.props.getBooleanProperty(FWProps.RESOLVER_PREFER_SB)) {
+      for (Pkg p : packages.values()) {
+        for (ExportPkg ep : p.exporters) {
+          if (ep.bpkgs.bg == bg) {
+            tempProvider.put(ep.name, ep);
+          }
+        }
+      }
+    }
     try {
       while (true) {
         tempCollision = null;
@@ -439,6 +467,7 @@ class Resolver {
           tempBlackList.addAll(baseTempBlackList);
           tempProvider.clear();
           tempRequired.clear();
+          tempAttached.clear();
           tempWires.clear();
         } else {
           break;
@@ -448,6 +477,7 @@ class Resolver {
       tempResolved = null;
       tempProvider = null;
       tempRequired = null;
+      tempAttached = null;
       tempBlackList = null;
       tempWires = null;
       resolveThread = null;
@@ -469,6 +499,15 @@ class Resolver {
   }
 
 
+  List<BundlePackages> getFragBundlePackages(BundlePackages fbpkgs)
+  {
+    if (tempAttached != null) {
+      return tempAttached.get(fbpkgs);
+    }
+    return null;
+  }
+
+
   /**
    * Get Pkg object for named package.
    *
@@ -698,7 +737,7 @@ class Resolver {
    */
   private boolean resolvePackages(Iterator<ImportPkg> pkgs, StringBuffer failReason)
       throws BundleException {
-    StringBuffer pkgFail = failReason != null ? new StringBuffer() : null;
+    StringBuffer pkgFail = failReason != null || framework.debug.resolver ? new StringBuffer() : null;
     boolean res = true;
     tempCollision = null;
     while (pkgs.hasNext()) {
@@ -712,16 +751,35 @@ class Resolver {
         framework.debug.println("resolvePackages: check - " + ip.pkgString());
       }
       List<ExportPkg> possibleProvider = new LinkedList<ExportPkg>();
+      int n_resolved = 0;
       for (ExportPkg ep : ip.pkg.exporters) {
+        int ss = framework.debug.resolver ? pkgFail.length() : 0;
         if (ip.checkAttributes(ep)) {
-          if (ip.bpkgs == ep.bpkgs || ip.checkPermission(ep)) {
-            possibleProvider.add(ep);
-          } else if (pkgFail != null) {
-            newFailReason(pkgFail, "No import permission", ep);
+          if (tempBlackList.contains(ep)) {
+            tempBlackListHits++;
+            if (pkgFail != null) {
+              newFailReason(pkgFail, "Package temporary black listed", ep);
+            }
+          } else {
+            if (ip.bpkgs == ep.bpkgs || ip.checkPermission(ep)) {
+              if (ep.bpkgs.isActive()) {
+                possibleProvider.add(n_resolved++, ep);
+              } else {
+                possibleProvider.add(ep);
+              }
+              continue;
+            } else if (pkgFail != null) {
+              newFailReason(pkgFail, "No import permission", ep);
+            }
           }
         } else if (pkgFail != null) {
           newFailReason(pkgFail, "Attributes don't match", ep);
         }
+        if (framework.debug.resolver) {
+          if (ip.pkg.providers.contains(ep)) {
+            framework.debug.println("resolvePackages: rejected provide because - " + pkgFail.substring(ss));
+          }
+        }
       }
       if (pkgFail != null) {
         if (possibleProvider.isEmpty() && pkgFail.length() == 0) {
@@ -766,9 +824,15 @@ class Resolver {
             if (pkgFail != null) {
               newFailReason(pkgFail, "Collied with previous selection", ep);
             }
+            if (framework.debug.resolver) {
+              framework.debug.println("resolvePackages: Provider black listed: " + ep);
+            }
             continue;
           }
           if (ep.zombie) {
+            if (framework.debug.resolver) {
+              framework.debug.println("resolvePackages: Provider zombie, skip: " + ep);
+            }
             continue;
           }
           final HashMap<String, ExportPkg> oldTempProvider = tempProviderClone();
@@ -782,6 +846,9 @@ class Resolver {
             if (pkgFail != null) {
               newFailReason(pkgFail, "Provider rejected because of uses directive ", ep);
             }
+            if (framework.debug.resolver) {
+              framework.debug.println("resolvePackages: Provider rejected because of uses directive: " + ep);
+            }
           }
         }
         if (provider == null) {
@@ -792,17 +859,22 @@ class Resolver {
         }
       }
       if (provider == null) {
-        if (ip.mustBeResolved()) {
-          res = false;
-          if (failReason != null) {
-            failReason.append(FWProps.NL);
-            failReason.append(ip.pkgString());
-            failReason.append(" -- ");
-            failReason.append(pkgFail);
-          }
-        } else {
-          if (framework.debug.resolver) {
-            framework.debug.println("resolvePackages: Ok, no provider for optional " + ip.name);
+        if (ip.internalOk == null) {
+          if (ip.mustBeResolved()) {
+            res = false;
+            if (failReason != null) {
+              failReason.append(FWProps.NL);
+              failReason.append(ip.pkgString());
+              failReason.append(" -- ");
+              failReason.append(pkgFail);
+            }
+            if (framework.debug.resolver) {
+              framework.debug.println("resolvePackages: Failed to find provider for, " + ip.name);
+            }
+          } else {
+            if (framework.debug.resolver) {
+              framework.debug.println("resolvePackages: Ok, no provider for optional, " + ip.name);
+            }
           }
         }
       }
@@ -845,20 +917,14 @@ class Resolver {
         }
         continue;
       }
-      if (ip.bpkgs == ep.bpkgs) {
+      if (ep.bpkgs.bg.isFragment()) {
+        continue;
+      }
+      if (ip.internalOk == null && ip.bpkgs.bg == ep.bpkgs.bg) {
         if (framework.debug.resolver) {
           framework.debug.println("pickProvider: internal wire ok for - " + ep);
         }
         ip.internalOk = ep;
-      } else if (!ep.checkPermission()) {
-        if (framework.debug.resolver) {
-          framework.debug.println("pickProvider: no export permission for - " + ep);
-        }
-        i.remove();
-        if (failReason != null) {
-          newFailReason(failReason, "No export permission for", ep);
-        }
-        continue;
       }
       if (ep.bpkgs.bg.bundle.state != Bundle.INSTALLED) {
         final HashMap<String, ExportPkg> oldTempProvider = tempProviderClone();
@@ -914,10 +980,11 @@ class Resolver {
         framework.debug.println("pickProvider: check possible provider - " + ep);
       }
       if (checkResolve(ep.bpkgs.bg, ep)) {
+        ExportPkg tep = ep.bpkgs.bg.isFragment() ? tempProvider.get(ep.name) : ep;
         if (framework.debug.resolver) {
-          framework.debug.println("pickProvider: " + ip + " - got provider - " + ep);
+          framework.debug.println("pickProvider: " + ip + " - got provider - " + tep);
         }
-        return ep;
+        return tep;
       }
       if (tempCollision != null  && savedCollision  == null) {
         // Save collision so that we can backtrack and try to avoid collision
@@ -966,51 +1033,112 @@ class Resolver {
       return true;
     }
     if (checkBundleSingleton(bg) == null) {
-      boolean retry;
-      final HashSet<ExportPkg> collisions = new HashSet<ExportPkg>();
-      do {
-        retry = false;
-        @SuppressWarnings("unchecked")
-        final HashSet<BundleGeneration> oldTempResolved = (HashSet<BundleGeneration>)tempResolved.clone();
+      List<BundleGeneration> fraghosts = checkFragmentHost(bg);
+      if (fraghosts == null) {
+        if (ep != null && !ep.checkPermission()) {
+          if (framework.debug.resolver) {
+            framework.debug.println("checkResolve: no export permission for - " + ep);
+          }
+          return false;
+        }
+        fraghosts = new ArrayList<BundleGeneration>(1);
+        fraghosts.add(null);
+      }
+      SavedTempState savedState = new SavedTempState();
+      SavedTempState savedState2 = savedState;
+      while (!fraghosts.isEmpty()) {
         if (!addTempResolved(bg)) {
           return false;
         }
-        final HashMap<String, ExportPkg> oldTempProvider = tempProviderClone();
-        @SuppressWarnings("unchecked")
-        final HashMap<RequireBundle, BundlePackages> oldTempRequired = (HashMap<RequireBundle, BundlePackages>)tempRequired.clone();
-        @SuppressWarnings("unchecked")
-        final HashSet<ExportPkg> oldTempBlackList = (HashSet<ExportPkg>)tempBlackList.clone();
-        tempBlackList.addAll(collisions);
-        final int oldTempWiresSize = tempWires.size();
-        if (ep != null) {
-          tempProvider.put(ep.pkg.pkg, ep);
-        }
-        final String breq = checkBundleRequirements(bg);
-        if (breq == null) {
-          if (checkRequireBundle(bg) == null) {
-            if (resolvePackages(bg.bpkgs.getImports(), null)) {
-              return true;
+        BundlePackages rbp;
+        ExportPkg rep;
+        BundleGeneration rbg = fraghosts.remove(0);
+        if (rbg != null) {
+          try {
+            if (framework.debug.resolver) {
+              framework.debug.println("checkResolve: fragment export of " + ep + " attach to:" + rbg);
             }
-            if (tempCollision != null) {
-              if (!oldTempProvider.containsValue(tempCollision)) {
-                collisions.add(tempCollision);
-                retry = true;
-                tempCollision = null;
+            if (rbg.bundle.isResolved()) {
+              rbp = new BundlePackages(rbg.bpkgs, bg.bpkgs);
+            } else {
+              if (checkResolve(rbg, null)) {
+                rbp = new BundlePackages(rbg.bpkgs, bg.bpkgs);
+                savedState2 = new SavedTempState();
+              } else {
+                continue;
+              }
+            }
+            rbp.registerPackages();
+            List<BundlePackages> fbps = tempAttached.get(bg.bpkgs);
+            if (fbps == null) {
+              fbps = new ArrayList<BundlePackages>();
+              tempAttached.put(bg.bpkgs, fbps);
+            }
+            fbps.add(rbp);
+          } catch (IllegalStateException ise) {
+            savedState.restore();
+            continue;
+          }
+          if (ep != null) {
+            rep = rbp.getExports(ep.name).next();
+            if (!rep.checkPermission()) {
+              if (framework.debug.resolver) {
+                framework.debug.println("checkResolve: no fragment export permission for - " + rep);
+              }
+              savedState.restore();
+              continue;
+            }
+          } else {
+            rep = null;
+          }
+        } else {
+          rbp = bg.bpkgs;
+          rep = ep;
+        }
+        final HashSet<ExportPkg> collisions = new HashSet<ExportPkg>();
+        boolean retry;
+        do {
+          retry = false;
+          tempBlackList.addAll(collisions);
+          if (rep != null) {
+            tempProvider.put(rep.pkg.pkg, rep);
+          }
+          final String breq = checkBundleRequirements(bg);
+          if (breq == null) {
+            if (checkRequireBundle(bg) == null) {
+              if (resolvePackages(rbp.getImports(), null)) {
+                  return true;
+              }
+              if (tempCollision != null) {
+                if (!savedState2.oldTempProvider.containsValue(tempCollision)) {
+                  collisions.add(tempCollision);
+                  retry = true;
+                  tempCollision = null;
+                }
               }
             }
           }
-        }
-        tempResolved = oldTempResolved;
-        tempProvider = oldTempProvider;
-        tempRequired = oldTempRequired;
-        tempBlackList = oldTempBlackList;
-        tempWires.subList(oldTempWiresSize, tempWires.size()).clear();
-      } while (retry);
+          if (retry) {
+            savedState2.restore();
+          }
+        } while (retry);
+      }
+      savedState.restore();
     }
     return false;
   }
 
 
+  private List<BundleGeneration> checkFragmentHost(BundleGeneration bg)
+  {
+    if (bg.isFragment()) {
+      return bg.fragment.targets();
+    } else {
+      return null;
+    }
+  }
+
+
   /**
    * Check that the packages that this provider uses do not collied with
    * previous selections. If a bundle doesn't have a uses directive we check all
@@ -1031,7 +1159,11 @@ class Resolver {
       final ArrayList<ExportPkg> checkList = new ArrayList<ExportPkg>();
       while (i.hasNext()) {
         final ImportPkg ip = i.next();
-        if (uses != null && !uses.contains(ip.pkg.pkg)) {
+        if (uses == null) {
+          if (!framework.props.getBooleanProperty(FWProps.RESOLVER_IMPLICIT_USES)) {
+            continue;
+          }
+        } else if (!uses.contains(ip.pkg.pkg)) {
           continue;
         }
         final ExportPkg ep = tempProvider.get(ip.pkg.pkg);
@@ -1040,8 +1172,15 @@ class Resolver {
                                   " with provider, " + ip.provider);
         }
         if (ep == null) {
-          tempProvider.put(ip.pkg.pkg, ip.provider);
-          checkList.add(ip.provider);
+          if (tempBlackList.contains(ip.provider)) {
+            if (framework.debug.resolver) {
+              framework.debug.println("checkUses: black listed provider for, " + ip.pkg.pkg);
+            }
+            return false;
+          } else {
+            tempProvider.put(ip.pkg.pkg, ip.provider);
+            checkList.add(ip.provider);
+          }
         } else if (ep != ip.provider) {
           if (framework.debug.resolver) {
             framework.debug.println("checkUses: mismatch in providers for, " + ip.pkg.pkg);
@@ -1078,18 +1217,16 @@ class Resolver {
       final List<BundleGeneration> bl = framework.bundles.getBundleGenerations(bg.symbolicName);
       if (bl.size() > 1) {
         if (framework.resolverHooks.hasHooks()) {
-          final BundleCapability bc = bg.getBundleCapability();
           Collection<BundleCapability> candidates = new LinkedList<BundleCapability>();
           List<BundleNameVersionCapability> active = new ArrayList<BundleNameVersionCapability>(bl.size());
           for (final BundleGeneration bg2 : bl) {
             if (bg2.singleton) {
               if (bg2 != bg) {
-                BundleNameVersionCapability bc2 = bg2.getBundleCapability();
-                if (bc2 != null) {
+                if (bg2.bundleCapability != null) {
                   if (bg2.bpkgs.isActive()) {
-                    active.add(bc2);
+                    active.add(bg2.bundleCapability);
                   } else {
-                    candidates.add(bc2);
+                    candidates.add(bg2.bundleCapability);
                   }
                 }
               }
@@ -1098,9 +1235,9 @@ class Resolver {
           if (!active.isEmpty()) {
             for (BundleNameVersionCapability abc : active) {
               Collection<BundleCapability> c = new LinkedList<BundleCapability>(candidates);
-              c.add(bc);
+              c.add(bg.bundleCapability);
               framework.resolverHooks.filterSingletonCollisions(abc, c);
-              if (c.contains(bc)) {
+              if (c.contains(bg.bundleCapability)) {
                 return abc.gen;
               } else {
                 candidates.removeAll(c);
@@ -1108,7 +1245,7 @@ class Resolver {
             }
           }
           if (!candidates.isEmpty()) {
-            framework.resolverHooks.filterSingletonCollisions(bc, candidates);
+            framework.resolverHooks.filterSingletonCollisions(bg.bundleCapability, candidates);
             for (final BundleCapability bc2 : candidates) {
               BundleGeneration bg2 = ((BundleRevisionImpl)bc2.getRevision()).gen;
               if (tempResolved.contains(bg2)) {
@@ -1336,56 +1473,66 @@ class Resolver {
     List<BundleImpl> resolve = new ArrayList<BundleImpl>(tempResolved.size());
     for (final BundleGeneration bg : tempResolved) {
       if (!bg.bpkgs.isActive()) {
-        for (final Iterator<ImportPkg> bi = bg.bpkgs.getImports(); bi.hasNext();) {
-          final ImportPkg ip = bi.next();
-          final ExportPkg ep = tempProvider.get(ip.name);
-          if (ep == null) {
-            // There could be an internal connection, that should export a
-            // package
-            if (ip.internalOk != null) {
-              if (ip.internalOk.pkg.providers.isEmpty() &&
-                  ip.internalOk.checkPermission()) {
-                ip.internalOk.pkg.addProvider(ip.internalOk);
-                if (framework.debug.resolver) {
-                  framework.debug.println("registerNewProviders: exported internal wire, "
-                                          + ip + " -> " + ip.internalOk);
-                }
-              } else {
-                if (framework.debug.resolver) {
-                  framework.debug.println("registerNewProviders: internal wire, "
-                                          + ip + " -> " + ip.internalOk);
-                }
-              }
-            }
-          } else {
-            // TODO! This check is already done, we should cache result
-            if (ip.checkAttributes(ep) && ip.checkPermission(ep)) {
-              ip.provider = ep;
-            } else {
-              // Check if got a missmatching internal wire.
+        List<BundlePackages> bps = tempAttached.get(bg.bpkgs);
+        if (bps == null) {
+          bps = Collections.singletonList(bg.bpkgs);
+        }
+        for (BundlePackages bp : bps) {
+          for (final Iterator<ImportPkg> bi = bp.getImports(); bi.hasNext();) {
+            final ImportPkg ip = bi.next();
+            final ExportPkg ep = tempProvider.get(ip.name);
+            if (ep == null) {
+              // There could be an internal connection, that should export a
+              // package
               if (ip.internalOk != null) {
-                if (ip.internalOk == ep) {
+                if (ip.internalOk.pkg.providers.isEmpty() &&
+                    ip.internalOk.checkPermission()) {
+                  ip.internalOk.pkg.addProvider(ip.internalOk);
                   if (framework.debug.resolver) {
-                    framework.debug.println("registerNewProviders: internal wire, " + ip + ", "
-                        + ep);
+                    framework.debug.println("registerNewProviders: exported internal wire, "
+                                            + ip + " -> " + ip.internalOk);
                   }
                 } else {
-                  // TODO, should we resolve when this happens!?
-                  framework.frameworkError(bg.bundle,
-                       new Exception("registerNewProviders: Warning! Internal wire for, " + ip +
-                                     ", does not match exported. " + ep));
+                  if (framework.debug.resolver) {
+                    framework.debug.println("registerNewProviders: internal wire, "
+                                            + ip + " -> " + ip.internalOk);
+                  }
+                }
+              }
+            } else {
+              // TODO! This check is already done, we should cache result
+              if (ip.checkAttributes(ep) && ip.checkPermission(ep)) {
+                ip.provider = ep;
+              } else {
+                // Check if got a missmatching internal wire.
+                if (ip.internalOk != null) {
+                  if (ip.internalOk == ep) {
+                    if (framework.debug.resolver) {
+                      framework.debug.println("registerNewProviders: internal wire, " + ip + ", "
+                          + ep);
+                    }
+                  } else {
+                    // TODO, should we resolve when this happens!?
+                    framework.frameworkError(bg.bundle,
+                         new Exception("registerNewProviders: Warning! Internal wire for, " + ip +
+                                       ", does not match exported. " + ep));
+                  }
                 }
               }
             }
           }
         }
         if (bg.bundle != bundle) {
-          resolve.add(bg.bundle);
+          if (bg.isFragment()) {
+            resolve.add(0, bg.bundle);
+          } else {
+            resolve.add(bg.bundle);
+          }
         }
       }
     }
     for (final BundleImpl b : resolve) {
-      if (b.getUpdatedState(null) == Bundle.INSTALLED) {
+      if (b.getUpdatedState(null, true) == Bundle.INSTALLED) {
         framework.frameworkError(b,
             new Exception("registerNewProviders: InternalError!"));
       }
@@ -1410,14 +1557,56 @@ class Resolver {
   private void checkThread() throws BundleException {
     Thread t = Thread.currentThread();
     if (t.equals(resolveThread)) {
-      throw new BundleException("Can not currently start new resolve during a resolve.");      
+      throw new BundleException("Can not currently start new resolve during a resolve.");
     }
     for (BundleGeneration bg : tempResolved) {
       if (bg.bundle.isBundleThread(t)) {
-		    throw new BundleException("Can not resolve a bundle inside current BundleListener." +
+        throw new BundleException("Can not resolve a bundle inside current BundleListener." +
                                   "Will cause a dead-lock. BID=" + bg.bundle.id);
       }
     }
   }
 
+
+  class SavedTempState {
+
+    final HashSet<BundleGeneration> oldTempResolved;
+    final HashMap<String, ExportPkg> oldTempProvider;
+    final HashMap<RequireBundle, BundlePackages> oldTempRequired;
+    final HashMap<BundlePackages, List<BundlePackages>> oldTempAttached;
+    final HashSet<ExportPkg> oldTempBlackList;
+    final int oldTempWiresSize;
+
+    @SuppressWarnings("unchecked")
+    SavedTempState()
+    {
+      oldTempResolved = (HashSet<BundleGeneration>)tempResolved.clone();
+      oldTempProvider = tempProviderClone();
+      oldTempRequired = (HashMap<RequireBundle, BundlePackages>)tempRequired.clone();
+      oldTempAttached = (HashMap<BundlePackages, List<BundlePackages>>)tempAttached.clone();
+      oldTempBlackList = (HashSet<ExportPkg>)tempBlackList.clone();
+      oldTempWiresSize = tempWires.size();
+    }
+
+    void restore() {
+      // Unregister temporary fragments
+      for (Entry<BundlePackages, List<BundlePackages>> e : tempAttached.entrySet()) {
+        List<BundlePackages> bps = e.getValue();
+        List<BundlePackages> oldbps = oldTempAttached.get(e.getKey());
+        if (oldbps != null) {
+          bps.removeAll(oldbps);
+        }
+        for (BundlePackages bp : bps) {
+          bp.unregisterPackages(true);
+        }
+      }
+      tempResolved = oldTempResolved;
+      tempProvider = oldTempProvider;
+      tempRequired = oldTempRequired;
+      tempAttached = oldTempAttached;
+      tempBlackList = oldTempBlackList;
+      tempWires.subList(oldTempWiresSize, tempWires.size()).clear();
+    }
+  }
+
 }
