@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, KNOPFLERFISH project
+ * Copyright (c) 2003-2016, KNOPFLERFISH project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,6 @@
 package org.knopflerfish.framework;
 
 import java.io.File;
-import java.util.List;
 import java.util.Vector;
 
 import org.osgi.framework.Bundle;
@@ -46,6 +45,8 @@ import org.osgi.framework.ServiceFactory;
 import org.osgi.framework.ServiceRegistration;
 import org.osgi.framework.startlevel.BundleStartLevel;
 import org.osgi.framework.startlevel.FrameworkStartLevel;
+import org.osgi.framework.startlevel.dto.BundleStartLevelDTO;
+import org.osgi.framework.startlevel.dto.FrameworkStartLevelDTO;
 import org.osgi.service.startlevel.StartLevel;
 
 
@@ -194,6 +195,9 @@ public class StartLevelController
 
   void setStartLevel(final int startLevel, final FrameworkListener... listeners)
   {
+    if (!bRun) {
+      throw new IllegalStateException("StartLevelService isn't running yet");
+    }
     fwCtx.perm.checkStartLevelAdminPerm();
     if (startLevel <= 0) {
       throw new IllegalArgumentException("Initial start level must be > 0, is "
@@ -269,10 +273,9 @@ public class StartLevelController
       }
       final Vector<BundleImpl> set = new Vector<BundleImpl>();
 
-      final List<BundleImpl> bundles = fwCtx.bundles.getBundles();
-
-      for (final BundleImpl bs : bundles) {
-        if (canStart(bs)) {
+      for (final BundleImpl bs : fwCtx.bundles.getBundles()) {
+        BundleArchive archive = bs.current().archive;
+        if (archive != null) {
           if (bs.getStartLevel() == currentLevel) {
             if (bs.current().archive.getAutostartSetting()!=-1) {
               set.addElement(bs);
@@ -285,21 +288,24 @@ public class StartLevelController
 
       for (int i = 0; i < set.size(); i++) {
         final BundleImpl bs = set.elementAt(i);
-        try {
-          if (bs.current().archive.getAutostartSetting()!=-1) {
-            if (fwCtx.debug.startlevel) {
-              fwCtx.debug.println("startlevel: start " + bs);
+        BundleArchive archive = bs.current().archive;
+        if (archive != null) {
+          try {
+            if (bs.current().archive.getAutostartSetting()!=-1) {
+              if (fwCtx.debug.startlevel) {
+                fwCtx.debug.println("startlevel: start " + bs);
+              }
+              int startOptions = Bundle.START_TRANSIENT;
+              if (isBundleActivationPolicyUsed(bs.current().archive)) {
+                startOptions |= Bundle.START_ACTIVATION_POLICY;
+              }
+              bs.start(startOptions);
             }
-            int startOptions = Bundle.START_TRANSIENT;
-            if (isBundleActivationPolicyUsed(bs.current().archive)) {
-              startOptions |= Bundle.START_ACTIVATION_POLICY;
-            }
-            bs.start(startOptions);
+          } catch (final IllegalStateException ignore) {
+            // Tried to start an uninstalled bundle, skip
+          } catch (final Exception e) {
+            fwCtx.frameworkError(bs, e);
           }
-        } catch (final IllegalStateException ignore) {
-          // Tried to start an uninstalled bundle, skip
-        } catch (final Exception e) {
-          fwCtx.frameworkError(bs, e);
         }
       }
     }
@@ -312,9 +318,7 @@ public class StartLevelController
 
       final Vector<BundleImpl> set = new Vector<BundleImpl>();
 
-      final List<BundleImpl> bundles = fwCtx.bundles.getBundles();
-
-      for (final BundleImpl bs : bundles) {
+      for (final BundleImpl bs : fwCtx.bundles.getBundles()) {
         if (bs.getState() == Bundle.ACTIVE ||
             (bs.getState() == Bundle.STARTING && bs.current().lazyActivation)) {
           if (bs.getStartLevel() == currentLevel + 1) {
@@ -544,7 +548,7 @@ public class StartLevelController
 
   }
 
-  BundleStartLevel bundleStartLevel(final BundleImpl bi) {
+  BundleStartLevelImpl bundleStartLevel(final BundleImpl bi) {
     return new BundleStartLevelImpl(this, bi);
   }
 
@@ -584,6 +588,16 @@ public class StartLevelController
       return st.isBundleActivationPolicyUsed(getBundleArchive());
     }
 
+    BundleStartLevelDTO getDTO()
+    {
+      BundleStartLevelDTO res = new BundleStartLevelDTO();
+      res.bundle = bi.id;
+      res.startLevel = getStartLevel();
+      res.activationPolicyUsed = isActivationPolicyUsed();
+      res.persistentlyStarted = isPersistentlyStarted();
+      return res;
+    }
+
     private BundleArchive getBundleArchive() {
       final BundleArchive res = bi.current().archive;
       if (res == null && bi.id != 0) {
@@ -594,9 +608,9 @@ public class StartLevelController
 
   }
 
-  FrameworkStartLevel frameworkStartLevel(final BundleImpl bi)
+  FrameworkStartLevelImpl frameworkStartLevel()
   {
-    return new FrameworkStartLevelImpl(this, bi);
+    return new FrameworkStartLevelImpl(this, fwCtx.systemBundle);
   }
 
   static class FrameworkStartLevelImpl
@@ -636,5 +650,13 @@ public class StartLevelController
     {
       st.setInitialBundleStartLevel(startlevel);
     }
+
+    FrameworkStartLevelDTO getDTO()
+    {
+      FrameworkStartLevelDTO res = new FrameworkStartLevelDTO();
+      res.startLevel = getStartLevel();
+      res.initialBundleStartLevel = getInitialBundleStartLevel();
+      return res;
+    }
   }
 }
