@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2015, KNOPFLERFISH project
+ * Copyright (c) 2003-2016, KNOPFLERFISH project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,6 +45,7 @@ import java.io.PrintStream;
 import java.io.Reader;
 import java.lang.reflect.Method;
 import java.net.URL;
+import java.util.ArrayList;
 import java.util.Dictionary;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -54,19 +55,28 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.SortedSet;
+import java.util.TreeMap;
 import java.util.TreeSet;
 import java.util.Vector;
 
 import org.osgi.framework.Bundle;
+import org.osgi.framework.BundleActivator;
+import org.osgi.framework.BundleContext;
 import org.osgi.framework.BundleEvent;
 import org.osgi.framework.BundleException;
 import org.osgi.framework.Constants;
 import org.osgi.framework.FrameworkEvent;
+import org.osgi.framework.FrameworkListener;
 import org.osgi.framework.Version;
 import org.osgi.framework.VersionRange;
+import org.osgi.framework.dto.BundleDTO;
+import org.osgi.framework.dto.FrameworkDTO;
+import org.osgi.framework.dto.ServiceReferenceDTO;
 import org.osgi.framework.launch.Framework;
 import org.osgi.framework.namespace.ExecutionEnvironmentNamespace;
+import org.osgi.framework.namespace.NativeNamespace;
 import org.osgi.framework.startlevel.FrameworkStartLevel;
+import org.osgi.framework.startlevel.dto.FrameworkStartLevelDTO;
 import org.osgi.framework.wiring.FrameworkWiring;
 
 /**
@@ -117,6 +127,7 @@ public class SystemBundle extends BundleImpl implements Framework {
 
   private FrameworkWiringImpl fwWiring;
 
+  private TreeMap<BundleImpl, BundleActivator> extensions = new TreeMap<BundleImpl, BundleActivator>();;
 
   /**
    * Construct the System Bundle handle.
@@ -130,10 +141,20 @@ public class SystemBundle extends BundleImpl implements Framework {
   /**
    * Initialize this framework.
    *
-   * @see org.osgi.framework.Framework#init
+   * @see org.osgi.framework.launch.Framework#init()
    */
   @Override
   public void init() throws BundleException {
+    init(new FrameworkListener [0]);
+  }
+
+  /**
+   * Initialize this framework and call listeners.
+   *
+   * @see org.osgi.framework.launch.Framework#init(FrameworkListener...)
+   */
+  @Override
+  public void init(FrameworkListener... listeners) throws BundleException {
     secure.checkExecuteAdminPerm(this);
 
     synchronized (lock) {
@@ -141,6 +162,7 @@ public class SystemBundle extends BundleImpl implements Framework {
 
       switch (state) {
       case INSTALLED:
+        state = RESOLVED;
       case RESOLVED:
         break;
       case STARTING:
@@ -149,7 +171,7 @@ public class SystemBundle extends BundleImpl implements Framework {
       default:
         throw new IllegalStateException("INTERNAL ERROR, Illegal state, " + state);
       }
-      doInit();
+      doInit(listeners);
     }
   }
 
@@ -389,10 +411,16 @@ public class SystemBundle extends BundleImpl implements Framework {
       res = fwWiring;
     } else if (FrameworkStartLevel.class.equals(type)) {
       if (fwCtx.startLevelController != null) {
-        res = fwCtx.startLevelController.frameworkStartLevel(this);
+        res = fwCtx.startLevelController.frameworkStartLevel();
       }
     } else if (Framework.class.equals(type)) {
       res = this;
+    } else if (FrameworkStartLevelDTO.class.equals(type)) {
+      if (fwCtx.startLevelController != null) {
+        res = fwCtx.startLevelController.frameworkStartLevel().getDTO();
+      }
+    } else if (FrameworkDTO.class.equals(type)) {
+      res = getFrameworkDTO();
     } else {
       // TODO filter which adaptation we can do?!
       res = adaptSecure(type);
@@ -439,28 +467,29 @@ public class SystemBundle extends BundleImpl implements Framework {
 
 
   /**
-   * Adds an bundle as an extension that will be included in the boot class path
-   * on restart.
-   * @throws BundleException Should not happened.
+   * Adds an bundle as an extension.
+   * TODO, check that bootclasspath extensions really are on the
+   * bootclasspath.
    */
   void attachExtension(BundleGeneration extension) throws BundleException  {
     if (extension.isBootClassPathExtension()) {
       // if we attach during startup, we assume that bundle is in BCP.
       if (getClassLoader() == null) {
-        current().attachFragment(extension);
+        current().attachFragment(extension, false);
       } else {
-        throw new UnsupportedOperationException(
-            "Bootclasspath extension can not be dynamicly activated");
+        throw new BundleException("Bootclasspath extension can not be dynamicly activated", BundleException.UNSUPPORTED_OPERATION);
       }
     } else {
+      if (extensions.containsKey(extension.bundle)) {
+        throw new BundleException("Framework extension updates can not be resolved", BundleException.STATECHANGE_ERROR);
+      }
       try {
         addClassPathURL(new URL("file:" + extension.archive.getJarLocation()));
-        current().attachFragment(extension);
-        handleExtensionActivator(extension);
       } catch (final Exception e) {
-        throw new UnsupportedOperationException(
-            "Framework extension could not be dynamicly activated, " + e);
+        throw new BundleException("Framework extension could not be dynamicly activated", BundleException.UNSUPPORTED_OPERATION, e);
       }
+      current().attachFragment(extension, false);
+      extensions.put(extension.bundle, handleExtensionActivator(extension));
     }
   }
 
@@ -520,21 +549,31 @@ public class SystemBundle extends BundleImpl implements Framework {
         fwCtx.props.getProperty(Constants.FRAMEWORK_SYSTEMPACKAGES));
     if (sp.length() == 0) {
       // Try the system packages file
-      addSysPackagesFromFile(sp, fwCtx.props.getProperty(FWProps.SYSTEM_PACKAGES_FILE_PROP), null);
-      if (sp.length() == 0) {
+      String sysPkgFile = fwCtx.props.getProperty(FWProps.SYSTEM_PACKAGES_FILE_PROP);
+      if (sysPkgFile.length() != 0) {
+        addSysPackagesFromFile(sp, sysPkgFile, null);
+      } else {
         // Try the system packages base property.
         sp.append(fwCtx.props.getProperty(FWProps.SYSTEM_PACKAGES_BASE_PROP));
 
         if (sp.length() == 0) {
           if (FWProps.androidApiLevel >= 0) {
-            addSysPackagesFromFile(sp, "android_packages.txt", new Version(FWProps.androidApiLevel, 0, 0));
+            addSysPackagesFromResource(sp, "android_packages.txt", new Version(FWProps.androidApiLevel, 0, 0));
           } else {
             // use default set of packages.
             String jver = fwCtx.props.getProperty(FWProps.SYSTEM_PACKAGES_VERSION_PROP);
 
-            Version jv;
+            Version jv = new Version(1, 7, 0);
             if (jver == null) {
-              jv = new Version(FWProps.javaVersionMajor, FWProps.javaVersionMinor, 0);
+              if (FWProps.javaVersionMajor >= 0) {
+                int minor = FWProps.javaVersionMinor > 0 ? FWProps.javaVersionMinor : 0;
+                jv = new Version(FWProps.javaVersionMajor, minor, 0);
+              } else {
+                if (fwCtx.debug.framework) {
+                  fwCtx.debug.println("No built in list of Java packages to be exported "
+                      + "by the system bundle for JRE with unknown version, using the list for " + jv);
+                }
+              }
             } else {
               try {
                 jv = new Version(jver);
@@ -542,19 +581,18 @@ public class SystemBundle extends BundleImpl implements Framework {
                 if (fwCtx.debug.framework) {
                   fwCtx.debug.println("No built in list of Java packages to be exported "
                       + "by the system bundle for JRE with version '" + jver
-                      + "', using the list for 1.7.");
+                      + "', using the list for " + jv);
                 }
-                jv = new Version(1,7,0);
               }
             }
-            addSysPackagesFromFile(sp, "packages.txt", jv);
+            addSysPackagesFromResource(sp, "packages.txt", jv);
           }
         } else {
           if (sp.charAt(sp.length() - 1) == ',') {
             sp.deleteCharAt(sp.length() - 1);
           }
         }
-        addSysPackagesFromFile(sp, "exports", null);
+        addSysPackagesFromResource(sp, "exports", null);
       }
     }
     final String extraPkgs = fwCtx.props.getProperty(Constants.FRAMEWORK_SYSTEMPACKAGES_EXTRA);
@@ -590,6 +628,7 @@ public class SystemBundle extends BundleImpl implements Framework {
     }
     gen.setWired();
     fwWiring = new FrameworkWiringImpl(fwCtx);
+    state = STARTING;
   }
 
 
@@ -607,6 +646,27 @@ public class SystemBundle extends BundleImpl implements Framework {
         }
       }
     }
+    exportPackageString = null;
+    provideCapabilityString = null;
+    generations.clear();
+    fwWiring = null;
+  }
+
+
+  BundleCapabilityImpl getNativeCapability() {
+    return current().getDeclaredCapabilities().get(NativeNamespace.NATIVE_NAMESPACE).get(0);
+  }
+
+
+  void extensionCallStart(BundleImpl b) {
+    if (b != null) {
+      callBundleActivatorStart(b);
+    } else {
+      BundleImpl[] bs = extensions.keySet().toArray(new BundleImpl[extensions.size()]);
+      for (BundleImpl be : bs) {
+        callBundleActivatorStart(be);
+      }
+    }
   }
 
   //
@@ -616,63 +676,92 @@ public class SystemBundle extends BundleImpl implements Framework {
   /**
    *
    */
-  private void doInit() throws BundleException {
-    state = STARTING;
+  private void doInit(FrameworkListener... listeners) throws BundleException {
     bootClassPathHasChanged = false;
-    fwCtx.init();
+    fwCtx.init(listeners);
   }
 
 
   /**
-   * Read a file with package names and add them to a stringbuffer. The file is
-   * searched for in the current working directory, then on the class path.
+   * Read a file with package names and add them to a stringbuffer.
    * Each line can have a java version guard at the end with format <tt>!VersionRange</tt>.
    *
    * @param sp Buffer to append the exports to. Same format as the
    *          Export-Package manifest header.
-   * @param sysPkgFile Name of the file to load packages to be exported from.
+   * @param sysPkgFile File name.
    * @param guard Version to test version guarded lines against.
    */
   private void addSysPackagesFromFile(StringBuffer sp, String sysPkgFile, Version guard) {
-    if (null == sysPkgFile || 0 == sysPkgFile.length()) {
-      return;
-    }
-
     if (fwCtx.debug.resolver) {
       fwCtx.debug.println("Will add system packages from file " + sysPkgFile);
     }
-
-    URL url = null;
     final File f = new File(new File(sysPkgFile).getAbsolutePath());
 
     if (!f.exists() || !f.isFile()) {
-      url = SystemBundle.class.getResource(sysPkgFile);
-      if (null == url) {
-        url = SystemBundle.class.getResource("/" + sysPkgFile);
+      throw new IllegalArgumentException("Could not add system bundle package exports from '"
+                                         + sysPkgFile + "', file not found.");
+    } else {
+      try {
+        addSysPackagesFromStream(sp, new FileReader(f), guard);
+      } catch (final IOException e) {
+        throw new IllegalArgumentException("Failed to read " + sysPkgFile + ": " + e);
+      } catch (final IllegalArgumentException e) {
+        throw new  IllegalArgumentException("Failed to parse " + sysPkgFile + ": " + e);
       }
-      if (null == url) {
-        if (fwCtx.debug.resolver) {
-          fwCtx.debug.println("Could not add system bundle package exports from '" + sysPkgFile
-              + "', file not found.");
-        }
+      if (fwCtx.debug.resolver) {
+        fwCtx.debug.println("\treading from " + f.getAbsolutePath());
       }
     }
+  }
+
+
+  /**
+   * Read a resource with package names and add them to a stringbuffer.
+   * Each line can have a java version guard at the end with format <tt>!VersionRange</tt>.
+   *
+   * @param sp Buffer to append the exports to. Same format as the
+   *          Export-Package manifest header.
+   * @param sysPkgResource Resource name.
+   * @param guard Version to test version guarded lines against.
+   */
+  private void addSysPackagesFromResource(StringBuffer sp, String sysPkgResource, Version guard) {
+    if (fwCtx.debug.resolver) {
+      fwCtx.debug.println("Will add system packages from resource " + sysPkgResource);
+    }
+    URL url = SystemBundle.class.getResource(sysPkgResource);
+    if (null == url) {
+      url = SystemBundle.class.getResource("/" + sysPkgResource);
+    }
+    if (null != url) {
+      try {
+        addSysPackagesFromStream(sp, new InputStreamReader(url.openStream()), guard);
+      } catch (final IOException e) {
+        throw new IllegalArgumentException("Failed to read resource " + sysPkgResource + ": " + e);
+      } catch (final IllegalArgumentException e) {
+        throw new  IllegalArgumentException("Failed to parse resource " + sysPkgResource + ": " + e);
+      }
+      if (fwCtx.debug.resolver) {
+        fwCtx.debug.println("\treading from " + url);
+      }
+    }
+  }
+
+
+  /**
+   * Read a stream with package names and add them to a stringbuffer.
+   * Each line can have a java version guard at the end with format <tt>!VersionRange</tt>.
+   *
+   * @param sp Buffer to append the exports to. Same format as the
+   *          Export-Package manifest header.
+   * @param reader Reader for stream to load packages to be exported from.
+   * @param guard Version to test version guarded lines against.
+   */
+  private void addSysPackagesFromStream(StringBuffer sp, Reader reader, Version guard)
+    throws IOException
+  {
     BufferedReader in = null;
     try {
-      Reader reader = null;
-      String source = null;
-
-      if (null == url) {
-        reader = new FileReader(f);
-        source = f.getAbsolutePath().toString();
-      } else {
-        reader = new InputStreamReader(url.openStream());
-        source = url.toString();
-      }
       in = new BufferedReader(reader);
-      if (fwCtx.debug.resolver) {
-        fwCtx.debug.println("\treading from " + source);
-      }
 
       String line;
       for (line = in.readLine(); line != null; line = in.readLine()) {
@@ -680,14 +769,12 @@ public class SystemBundle extends BundleImpl implements Framework {
         if (line.length() > 0 && !line.startsWith("#")) {
           int idx = line.lastIndexOf('!');
           if (idx != -1) {
-            try {
-              if (new VersionRange(line.substring(idx +1)).includes(guard)) {
-                line = line.substring(0, idx);
-              } else {
-                // Not in version range skip.
-                continue;
-              }
-            } catch (IllegalArgumentException _ignore) { }
+            if (new VersionRange(line.substring(idx +1)).includes(guard)) {
+              line = line.substring(0, idx);
+            } else {
+              // Not in version range skip.
+              continue;
+            }
           }
           if (sp.length() > 0) {
             sp.append(",");
@@ -695,8 +782,6 @@ public class SystemBundle extends BundleImpl implements Framework {
           sp.append(line);
         }
       }
-    } catch (final IOException e) {
-      throw new IllegalArgumentException("Failed to read " + sysPkgFile + ": " + e);
     } finally {
       try {
         in.close();
@@ -917,6 +1002,7 @@ public class SystemBundle extends BundleImpl implements Framework {
         stopAllBundles();
         saveClasspaths();
       }
+      extensionCallStop();
       synchronized (lock) {
         fwCtx.uninit();
         shutdownThread = null;
@@ -974,10 +1060,8 @@ public class SystemBundle extends BundleImpl implements Framework {
       }
     }
 
-    final List<BundleImpl> allBundles = fwCtx.bundles.getBundles();
-
     // Set state to INSTALLED and purge any unrefreshed bundles
-    for (final BundleImpl bundleImpl : allBundles) {
+    for (final BundleImpl bundleImpl : fwCtx.bundles.getBundles()) {
       final BundleImpl b = bundleImpl;
       if (b.getBundleId() != 0) {
         b.setStateInstalled(false);
@@ -1045,17 +1129,79 @@ public class SystemBundle extends BundleImpl implements Framework {
    *
    * @param extension the extension bundle to process.
    */
-  private void handleExtensionActivator(final BundleGeneration extension) {
+  private BundleActivator handleExtensionActivator(final BundleGeneration extension) throws BundleException {
     String extActivatorName =
-      extension.archive.getAttribute("Extension-Activator");
+      extension.archive.getAttribute(Constants.EXTENSION_BUNDLE_ACTIVATOR);
     extActivatorName = null!=extActivatorName ? extActivatorName.trim() : null;
 
-    if (null!=extActivatorName && extActivatorName.length()>0) {
-      fwCtx.log("Activating extension: " + extension.symbolicName
+    if (null != extActivatorName && extActivatorName.length() > 0) {
+      fwCtx.log("Create bundle activator for extension: " + extension.symbolicName
                 + ":" +extension.version + " using: " +extActivatorName);
-      fwCtx.activateExtension(extension);
+      try {
+        final Class<BundleActivator> c = (Class<BundleActivator>)Class.forName(extActivatorName);
+        return c.newInstance();
+      } catch (Throwable t) {
+        final String msg = "Failed to instanciate extension activator " + extActivatorName + ", " + extension.bundle;
+        fwCtx.log(msg, t);
+        throw new BundleException(msg, BundleException.ACTIVATOR_ERROR, t);
+      }
+    }
+    return null;
+  }
+
+
+  private void callBundleActivatorStart(BundleImpl b) {
+    BundleActivator ba = extensions.get(b);
+    if (ba != null) {
+      if (fwCtx.debug.framework) {
+        fwCtx.debug.println("Call extension bundle start: " + b);
+      }
+      try {
+        ba.start(bundleContext);
+      } catch (final Throwable t) {
+        extensions.put(b, null);
+        final String msg = "Failed to start framework extension, " + b;
+        fwCtx.frameworkError(b, new BundleException(msg, BundleException.ACTIVATOR_ERROR, t));
+      }
     }
   }
 
 
+  private void extensionCallStop() {
+    BundleImpl[] bs = extensions.keySet().toArray(new BundleImpl[extensions.size()]);
+    for (int i = bs.length - 1; i >= 0; i--) {
+      if (fwCtx.debug.framework) {
+        fwCtx.debug.println("Call extension bundle stop: " + bs[i]);
+      }
+      BundleActivator ba = extensions.get(bs[i]);
+      if (ba != null) {
+        try {
+          ba.stop(bundleContext);
+        } catch (final Throwable t) {
+          final String msg = "Failed to stop framework extension, " + bs[i];
+          fwCtx.frameworkError(bs[i], new BundleException(msg, BundleException.ACTIVATOR_ERROR, t));
+        }
+      }
+    }
+    extensions.clear();
+  }
+
+
+  private FrameworkDTO getFrameworkDTO() {
+    FrameworkDTO res = new FrameworkDTO();
+    res.bundles = new ArrayList<BundleDTO>();
+    for (BundleImpl bi : fwCtx.bundles.getBundles()) {
+      res.bundles.add(bi.getDTO());
+    }
+    res.properties = fwCtx.props.getFWProperties();
+    res.services = new ArrayList<ServiceReferenceDTO>();
+    for (ServiceRegistrationImpl sri : fwCtx.services.getAllRegistered()) {
+      ServiceReferenceDTO srdto = sri.getDTO();
+      if (srdto != null) {
+        res.services.add(srdto);
+      }
+    }
+    return res;
+  }
+
 }
